package ui {
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	import flash.text.AntiAliasType;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	/**
	 * @author bram
	 */
	public class TimeDisplay extends Sprite
	{
		[Embed(source='../../media/time_background.png')]
		private var BackgroundImage : Class;

		private var _duration : Number;
		private var _previousTime : Number;
		private var _inverseTime : Boolean;

		private var _text : TextField;

		private var _background : Bitmap;
		private var _showMiliseconds : Boolean;

		public function TimeDisplay(large : Boolean)
		{
			buttonMode = true;
			_showMiliseconds = large;
			
			if (large)
			{
				_background = new BackgroundImage();
				_background.width = 100;
				addChild(_background);
			}
        	
			var textFormat:TextFormat = new TextFormat();
			textFormat.size = large ? 14 : 11;
			textFormat.color = 0xffffff;
			textFormat.font = "VeraMono";
			textFormat.align = TextFormatAlign.LEFT;

			_text = new TextField();
			_text.embedFonts = true;
			_text.defaultTextFormat = textFormat;
			_text.textColor = 0xffffff;
			_text.antiAliasType = AntiAliasType.ADVANCED;
			_text.mouseEnabled = false;

			if (large)
			{
				_text.height = 20;
				_text.width = 90;
				_text.x = 4;
				_text.y = 9;
			}
			else
			{
				_text.width = 48;
				_text.height = 14;
				_text.x = 0;
				_text.y = -1;
				this.graphics.beginFill(0, 0.2);
				this.graphics.drawRect(0, 0, 46, 16);
			}

            
			addChild(_text);
            
			addEventListener(MouseEvent.MOUSE_DOWN, onTimeDisplayClicked);
            
			_inverseTime = true;
			_previousTime = 0;
		}

		public function get duration() : Number
		{
			return _duration;
		}

		public function set duration(duration : Number) : void
		{
			_duration = duration;
			update();
		}

		public function update(time : Number = -1) : void
		{
			if (time < 0)
        		time = _previousTime;
        	
			if (_inverseTime)
        		_text.text = "-" + TimeDisplay.timeToString(_duration - time, _showMiliseconds);
        	else
        		_text.text = " " + TimeDisplay.timeToString(time, _showMiliseconds);

			_previousTime = time;
		}

		public function onTimeDisplayClicked(e : MouseEvent) : void
		{
			_inverseTime = !_inverseTime;
			update();
			e.stopImmediatePropagation();
		}

		public static function timeToString(time : Number, addMiliseconds : Boolean) : String
		{
			var minutes : int = time / (60 * 1000);
			time -= minutes * 60 * 1000;
			var seconds : int = time / (1000);
			time -= seconds * 1000;
			var miliseconds : int = time;
			
			var display : String = "";
			
			if (minutes < 10)
				display += "0";
			display += minutes.toString() + ":";
        	
			if (seconds < 10)
        		display += "0";
			display += seconds.toString();
			
			if (addMiliseconds)
			{
				display += ":";
				if (miliseconds < 10)
	        		display += "00";
	        	else if (miliseconds < 100)
	        		display += "0";
				display += miliseconds.toString();
			}
						
			return display;        	
		}
	}
}
