# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-05-09 15:25
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
from django.db.models import Count
from utils.mail import transform_unique_email


def populate_sameuser(apps, schema_editor):
    User = apps.get_model("auth", "User")
    SameUser = apps.get_model("accounts", "SameUser")

    # Get all email addresses which are used in more than one account
    email_addresses = User.objects.values_list("email", flat=True)\
        .annotate(num_email=Count('email'))\
        .filter(num_email__gt=1).exclude(email="")

    for email in email_addresses:
        # Get all users with that email address
        uss = User.objects.filter(email=email).order_by('-last_login').all()

        # Check that there are 2 users (there should be no more than 2 users with the same address)
        assert uss.count() == 2

        # Assign users to variables and modify email of second user (to prevent the duplicates)
        u1 = uss[0]
        u2 = uss[1]
        u2.email = transform_unique_email(email)
        u2.save()

        # Create SameUser object
        SameUser.objects.create(main_user=u1, main_orig_email=email,
                                secondary_user=u2, secondary_orig_email=email)


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('accounts', '0008_auto_20170331_1702'),
    ]

    operations = [
        migrations.CreateModel(
            name='SameUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('main_orig_email', models.CharField(max_length=200)),
                ('secondary_orig_email', models.CharField(max_length=200)),
                ('main_user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('secondary_user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.RunPython(populate_sameuser, migrations.RunPython.noop)
    ]
