"""
This was copied directl from django, but a small hack was added to allow different configuration of
markdown. It uses MARKDOWN_FILTER_SETTINGS from settings.py
"""

from django import template
from django.conf import settings
from django.utils.encoding import smart_str, force_unicode
from django.utils.safestring import mark_safe

register = template.Library()


def markdown(value, arg=''):
    """
    Runs Markdown over a given value, optionally using various
    extensions python-markdown supports.

    Syntax::

        {{ value|markdown:"extension1_name,extension2_name..." }}

    To enable safe mode, which strips raw HTML and only returns HTML
    generated by actual Markdown syntax, pass "safe" as the first
    extension in the list.

    If the version of Markdown in use does not support extensions,
    they will be silently ignored.

    """
    try:
        import markdown
    except ImportError:
        if settings.DEBUG:
            raise template.TemplateSyntaxError, "Error in {% markdown %} filter: The Python markdown library isn't installed."
        return force_unicode(value)
    else:
        # markdown.version was first added in 1.6b. The only version of markdown
        # to fully support extensions before 1.6b was the shortlived 1.6a.
        if hasattr(markdown, 'version'):
            extensions = [e for e in arg.split(",") if e]
            if len(extensions) > 0 and extensions[0] == "safe":
                extensions = extensions[1:]
                safe_mode = True
            else:
                safe_mode = False

            extension_configs = {'wikilinks': [('base_url', '/help/'), ('end_url', '/'), ('html_class', 'wikilink')]}
             
            md = markdown.Markdown(extensions=extensions, extension_configs=extension_configs, safe_mode=safe_mode)
            
            # Unicode support only in markdown v1.7 or above. Version_info
            # exist only in markdown v1.6.2rc-2 or above.
            if getattr(markdown, "version_info", None) < (1, 7):
                return mark_safe(force_unicode(md.convert(force_unicode(value))))
            else:
                return mark_safe(md.convert(force_unicode(value)))
        else:
            return mark_safe(force_unicode(markdown.markdown(smart_str(value))))


markdown.is_safe = True

register.filter(markdown)
